// Author: Heinrich Moser, 9925866, http://www.heinzi.at
//
// Accesses the Google Webservice using
// automatically created stubs
//
// See googleclient.html for a more detailed description

package at.heinzi.googleclient;

import java.io.*;
import at.heinzi.googleclient.stubs.*;

public class StubClient {

    static BufferedReader stdin = new BufferedReader(new InputStreamReader(System.in));
    static final int SEARCH = 1;
    static final int CACHE = 2;
    static final int SPELL = 3;
    static final String KEY = "Jy5IbvlQFHJsV+Nz/azR4K4Le7O+s9hq";

    public static void main(String[] args) {
        System.out.println("StubClient, Version 1.0");
        (new StubClient()).run(args);
    }

    public void run(String[] args) {
        // Ask for request type
        int type = p_readInt("(1) Search, (2) retrieve cached page, (3) spell check");

        switch (type) {
        case SEARCH:
            this.p_doSearch();
            break;
        case CACHE:
            this.p_doCache();
            break;
        case SPELL:
            this.p_doSpellCheck();
            break;
        default:
        }
    }

    // Search the web
    private void p_doSearch() {

        // Ask for search terms
        String q = p_readString("Search terms");
        int start = 1; // Start with first result

        // Ask to filter similar results
        boolean filter = p_readBoolean("Filter similar results");

        // Ask for country restrictions
        String restrict = p_readString("(Optional) Country restrictions (e.g. \"countryAT\", \"countryEU\", ...)");

        // Ask to filter adult content
        boolean safeSearch = p_readBoolean("Exclude adult content");
        
        // Ask for language restrictions
        String lr = p_readString("(Optional) Language restrictions (e.g. \"lang_de\", \"lang_en\", ...)");

        while (start != 0) {
            GoogleSearchResult r = null;

            // Get results
            try {
                GoogleSearchPort p = new GoogleSearchService_Impl().getGoogleSearchPort();
                r = p.doGoogleSearch(KEY, q, start, 10, filter, restrict, safeSearch, lr, "", "");
            }  catch (java.rmi.RemoteException ex) {
                ex.printStackTrace();
                System.exit(1);
            }
            
            // Print list of results
            System.out.println("Results " + Integer.toString(start) + " to " + Integer.toString(start + 10));
            System.out.println("");
            for (int i = 0; i < r.getResultElements().length; i++) {
                ResultElement re = r.getResultElements()[i];
                System.out.println("(" + Integer.toString(i) + ") " + re.getTitle());
                System.out.println(re.getURL());
                System.out.println("");
            }
            
            // Ask for action
            while (true) {
                String action = p_readString("(Num) Details, (n) next results, (p) previous results, (q) quit");
                if (action.equals("n")) {  // next 10 results
                    start = start + 10;
                    break;
                } else if (action.equals("p")) {  // previous 10 results
                    start = (start < 11) ? 1 : (start - 10);
                    break;
                } else if (action.equals("q")) {  // quit
                    start = 0;
                    break;
                } else {  // show details
                    int i;
                    try {
                        i = Integer.parseInt(action);
                    } catch (NumberFormatException ex) {
                        i = -1;
                    }
                    if ((i >= 0) && (i < r.getResultElements().length)) {
                        this.p_showDetails(r.getResultElements()[i]);
                    }
                }
            } 
        } // loop while (start != 0)
    }

    // Get cached page
    private void p_doCache() {
        String url = p_readString("URL");
        try {
            GoogleSearchPort p = new GoogleSearchService_Impl().getGoogleSearchPort();
            byte[] cachedPage = p.doGetCachedPage(KEY, url);
            System.out.println(new String(cachedPage));
        }  catch (java.rmi.RemoteException ex) {
            ex.printStackTrace();
            System.exit(1);
        }
    }
   
    // Get spell checking suggestion
    private void p_doSpellCheck() {
        String s = p_readString("Words to check");
        try {
            GoogleSearchPort p = new GoogleSearchService_Impl().getGoogleSearchPort();
            String checked = p.doSpellingSuggestion(KEY, s);
            System.out.println("Spelling suggestion: " + checked);
        }  catch (java.rmi.RemoteException ex) {
            ex.printStackTrace();
            System.exit(1);
        }
    }

    // Show the details of a result element
    private void p_showDetails(ResultElement re) {
        System.out.println("Title: " + re.getTitle());
        System.out.println("URL: " + re.getURL());
        System.out.println("Cached size: " + re.getCachedSize());
        if (re.getDirectoryCategory() != null) {
            System.out.println("Directory category: " + re.getDirectoryCategory().getFullViewableName());
        }
        System.out.println("Directory title: " + re.getDirectoryTitle());
        System.out.println("Host name: " + re.getHostName());
        System.out.println("Snippet: \n" + re.getSnippet() + "\n");
        System.out.println("Summary: \n" + re.getSummary() +  "\n");
        System.out.println("");
    }
    
    // reads an Integer from the command line
    private static int p_readInt(String prompt) {
        String s;
        int i = 0;
        boolean done = false;

        while (!done) {
            try {
                System.out.print(prompt + ": ");
                s = stdin.readLine();
            } catch (IOException ex) {
                System.err.println("IOException: " + ex.getMessage());
                System.exit(1);
                return 0; /* to avoid compiler warning */
            }
            try {
                i = Integer.parseInt(s);
                done = true;
            } catch (NumberFormatException ex) {
                System.out.println("Please enter a valid integer!");
                done = false;
            }
        }
        return i;
    }

    // reads a Boolean from the command line
    private static boolean p_readBoolean(String prompt) {
        try {
            System.out.print(prompt + " (y/n) [n]: ");
            String s = stdin.readLine();
            return s.equals("y") || s.equals("Y");
        } catch (IOException ex) {
            System.err.println("IOException: " + ex.getMessage());
            System.exit(1);
            return false; /* to avoid compiler warning */
        }
    }

    // reads a String from the command line
    private static String p_readString(String prompt) {
        try {
            System.out.print(prompt + ": ");
            return stdin.readLine();
        } catch (IOException ex) {
            System.err.println("IOException: " + ex.getMessage());
            System.exit(1);
            return null; /* to avoid compiler warning */
        }
    }
        
}
